/*************************************************************************************************************************************************

File: 		Cold6main.c
Author:		Gordon D. Dick, P. Eng. (retired)

Description: This is the final version of the PIC-based Cold Room Temperature Contol code.

Apr 22, 2015: Last date this code was worked on. Satisfactory operation in both TEST and NORMAL modes. Revision history is shown in previous
versions but not included here.


*************************************************************************************************************************************************/



/*					This code was developed using MPLAB v8.83 and the HITECH C v9.65 free compiler for use with a PIC16F876(A).					*/




/*************************************************************************************************************************************************
																	Includes
*************************************************************************************************************************************************/
#include	<htc.h>						//generic include for all devices
#include	"LCD.h"
#include	"Onewire.h"

//NOTE:****************************
//Use the Configure tab to select: HS oscillator, WDT disabled,  BOR enabled, Power-up timer enabled, RB3 is digital IO, code protection off,
//unprotected program memory may be written to by EECON. Configuration word is 0x3F72.
//*********************************


/*************************************************************************************************************************************************
																	Defines
*************************************************************************************************************************************************/
#define FOSC 4000000
// Port I/O directions (0 = output, 1 = input)
//Port B directions defined in LCD.h
#define PORTC_DIR	0b11111000				//RC0 direction(0= CW ie toward Home, 1=CCW), RC1 step, RC2 enable, RC3 Test input normally high,
											//RC7 home input normally high
#define TTmax 52							//Test mode max temp ( 26C )
#define TTmin 36							//Test mode min temp ( 18C )
#define NTmax 20							//Normal mode max temp (10C )
#define NTmin 4								//Normal mode min temp ( 2C )



/*************************************************************************************************************************************************
													I/O Definitions
*************************************************************************************************************************************************/
/*					
					Need 4 outputs for LCD data:	RB4 : RB7
					Need 2 outputs for LCD cntrl:	RB1 : RB2		RB1 = RS,	RB2 = CS
					Need 3 outputs for Motor cntrl:	RC0	: RC2		RC0 = DIR,	RC1 = STEP,	RC2 = ENABLE
					Need 1 input for Test :			RC3
					Need 1 input for Homing	:		RC7
					Need 1 input for Onewire:		RB0

*/



/*************************************************************************************************************************************************
															GLOBAL VARIABLES
*************************************************************************************************************************************************/
	char ucTicCount = 0;
	char ucTicFlag = 0;											//indicates a new tick when set
	char ucPass = 0;											//pass counter for activity indicator
	unsigned char ucActualPosition;
	unsigned char ucDesiredPosition;
	unsigned char ucMins = 0;


/*************************************************************************************************************************************************
															GLOBAL CONSTANTS
*************************************************************************************************************************************************/
	const unsigned int uiWeightTable[] = { 10000, 1000, 100, 10, 1 };
	const char cBanner1[] 	=	" PIC Cold Room  ";			//16 char strings
	const char cBanner2[]	=	"   Control by   ";
	const char cBanner3[]	=	" G. D. Dick, EE ";
	const char cBanner4[]	=	"Rev 2.0 Mar 2015";
	const char cStatic1[]	=	"OldTemp:xx.x    ";
	const char cStatic2[]	=	"Tmp:xx.x P:xxx  ";
	const char cMess1[]		=	" Motor is homing";
	const char cMess2[]		=	" Homing failed! ";
	const char cMess3[]		=	" Jumper is out. ";
	const char cMess4[]		=	"  Mode = NORMAL ";
	const char cMess5[]		=	"  Jumper is in. ";
	const char cMess6[]		=	"   Mode = TEST  ";
	const char cBlank3[]	=	"   ";
	const char cBlank4[]	=	"    ";
	const char cBlankLine[] =	"                ";



/*************************************************************************************************************************************************
																Prototypes
*************************************************************************************************************************************************/
	void Timer0_ISR( void );



/*************************************************************************************************************************************************
																	Main
*************************************************************************************************************************************************/
void main(void)
{
	char cTemp = 0;
	unsigned int uiOldTemp = 0;
	unsigned char ucPassOld = 0;
	unsigned char ucMinsOld = 0;
	unsigned int uiTemperature;
	char cTempBuffer[6] = { 0 };
	char cPosBuffer[4] = { 0 };
	unsigned char ucTmaxCount = 0;
	unsigned char ucTminCount = 0;
	unsigned int uiCircBuff[8] = { 0 };
	unsigned char ucCircBuffIndex = 0;
	unsigned int uiCircBuffSum = 0;
	unsigned char ucModeFlag = 0;


	TRISC = PORTC_DIR;
	TRISB = PORTB_DIR;

	OPTION_REG = 0x05;													//TMR0 uses internal clock, /64, this gives a 16ms tick interval
	INTCON |= 0x20;														//enable TMR0 ints

	__delay_ms( 1000 );													//for the lcd
	LCDInit();
	LCDConstString2Spot( 1, 0, cBanner1 );								//cBanner1[] = " PIC Cold Room  "
	LCDConstString2Spot( 2, 0, cBanner2 );								//cBanner2[] = "   Control by   "
	__delay_ms( 3000 );
	LCDConstString2Spot( 1, 0, cBanner3 );								//cBanner3[] = " G. D. Dick, EE "
	LCDConstString2Spot( 2, 0, cBanner4 );								//cBanner4[] = "Rev 2.0 Mar 2015"
	__delay_ms( 3000 );


	LCDConstString2Spot( 1, 0, cMess1 );								//cMess1[] = " Motor is homing"
	LCDConstString2Spot( 2, 0, cBlankLine );
	PORTCbits.RC2 = 0;													//enable EasyDriver
	PORTCbits.RC0 = 0;													//establish CW direction
	while( PORTCbits.RC7 && (cTemp < 250) )								//Home switch is normally open
	{
		PORTCbits.RC1 = 1;
		PORTCbits.RC1 = 0;												//generate a step pulse
		__delay_ms( 20 );												//pause a little
		cTemp++;														//				and count a step
	}
	if( cTemp >= 250 )
	{
		LCDConstString2Spot( 2, 0, cMess2 );							//cMess2[] = " Homing failed! "
		while(1);														//lock the micro up here if Homing failed
	}
	PORTCbits.RC2 = 1;													//disable EasyDriver
	ucActualPosition = 0;												//This is now 0.


//Check the Test jumper to see which mode should be used.
	if( PORTCbits.RC3 )													//RC3 is normally high with no jumper ie Normal Mode
		ucModeFlag = 1;
	else
		ucModeFlag = 0;													//RC3 is low when the jumper is inserted ie Test Mode

//Show which mode is being used.
	if( ucModeFlag )													//this NORMAL mode
//********************************************************************* ST Mode starts here
	{
		LCDConstString2Spot( 1, 0, cMess3 );							//cMess3[] =	" Jumper is out. ";
		LCDConstString2Spot( 2, 0, cMess4 );							//cMess4[] =	"  Mode = NORMAL ";
		__delay_ms( 3000 );
	}
	else
	{
		LCDConstString2Spot( 1, 0, cMess5 );							//cMess5[] =	"  Jumper is in. "
		LCDConstString2Spot( 2, 0, cMess6 );							//cMess6[] =	"   Mode = TEST  "
		__delay_ms( 3000 );
	}

//Paint the static screen headers.
	LCDConstString2Spot( 1, 0, cStatic1 );								//cStatic1[] = "OldTemp:xx.x    "
	LCDConstString2Spot( 2, 0, cStatic2 );								//cStatic2[] = "Tmp:xx.x P:xxx  "

	INTCON |= 0x80;														//enable global ints


//The main loop begins here.
  while(1)
  {
		while( ucTicFlag == 0 );										//hang the processor here until a new tick arrives
		ucTicFlag = 0;													//clear the flag for next time

	if( !ucModeFlag )
//********************************************************************* TEST Mode starts here
	{
//This code executes once per second.

		if( ucPass != ucPassOld )
		{
			xtoa( ucActualPosition, uiWeightTable, cPosBuffer );
			LCDConstString2Spot( 2, 11, cBlank4 );						//erase
			LCDConstString2Spot( 2, 11, cPosBuffer );					//show actual position

			uiTemperature = GetData();									//get a new temperature sample

			if( uiTemperature < TTmin )									//make sure the temperature used is bounded
				uiTemperature = TTmin;
			if( uiTemperature > TTmax )
				uiTemperature = TTmax;

			xtoa( uiTemperature >> 1, uiWeightTable, cTempBuffer );
			LCDConstString2Spot( 2, 4, cTempBuffer );					//show the whole part of temperature
			LCDConstString2Spot( 2, 6, cBlank3 );
			CursorXY( 6, 2 );
			LCDPutc( '.' );

			if( uiTemperature & 0x0001 )
				LCDPutc( '5' );
			else
				LCDPutc( '0' );

			if( uiTemperature >= (uiOldTemp + 2) )
			{
				ucDesiredPosition = 25 * (uiTemperature >> 1) - 450;	//calculate a new desired position
				uiOldTemp = uiTemperature;								//save a copy of this temp for next pass
				xtoa( uiOldTemp >> 1, uiWeightTable, cTempBuffer );
				LCDString2Spot( 1, 8, cTempBuffer );					//show the whole part of measured temperature
					CursorXY( 11, 1 );
				if( uiTemperature & 0x0001 )							//show the fractional part
					LCDPutc( '5' );
				else
					LCDPutc( '0' );
			}//end of if( uiTemperature >= (uiOldTemp + 2) )

//If the temperature is at TTmax for 5 consecutive passes, open the flap to max.
			if( uiTemperature >= TTmax )
			{
				ucTmaxCount++;
				if( ucTmaxCount >= 4 )									//5 consecutive passes yet?
				{
					ucDesiredPosition = 200;							//If Yes, force flap fully open.
					ucTmaxCount = 4;									//don't count passes past 4
				}
			}	
			else
				ucTmaxCount = 0;

			if( uiTemperature <= (uiOldTemp - 3) )
			{
				ucDesiredPosition = 25 * (uiTemperature >> 1) - 450;	//calculate a new desired position
				uiOldTemp = uiTemperature;								//save a copy of this temp for next pass
				xtoa( uiOldTemp >> 1, uiWeightTable, cTempBuffer );
				LCDString2Spot( 1, 8, cTempBuffer );					//show the whole part of old temperature
				CursorXY( 11, 1 );
				if( uiTemperature & 0x0001 )							//show the fractional part
					LCDPutc( '5' );
				else
					LCDPutc( '0' );
			}//end of if( uiTemperature <= (uiOldTemp - 3) )

//If the temperature is at TTmin for 5 passes, close the flap.
			if( uiTemperature <= TTmin )
			{
				ucTminCount++;
				if( ucTminCount >= 4 )									//5 consecutive passes yet?
				{
					ucDesiredPosition = 0;								//If Yes, force flap closed.
					ucTminCount = 4;									//don't count passes past 4
				}
			}	
			else
				ucTminCount = 0;
			
			ucPassOld = ucPass;											//update the pass count
			Blink( ucPass );											//					and blink the symbol

		}//end of if( ucPass != ucPassOld )
  }//end of if( !ucModeFlag )
 
//********************************************************************* NORMAL Mode starts here
  else
  {
//This code executes once per second.

		if( ucPass != ucPassOld )
		{
			ucPassOld = ucPass;											//update the old pass count
			xtoa( ucActualPosition, uiWeightTable, cPosBuffer );
			LCDConstString2Spot( 2, 11, cBlank4 );						//erase
			LCDString2Spot( 2, 11, cPosBuffer );						//show actual position
			xtoa( ucPass, uiWeightTable, cPosBuffer );
			LCDConstString2Spot( 1, 14, cBlank3 );						//erase
			LCDString2Spot( 1, 14, cPosBuffer );						//show pass count


		}
//This code executes once per minute.
		if( ucMins != ucMinsOld )
		{
			uiTemperature = GetData();									//get a new temperature sample

			if( uiTemperature < NTmin )									//make sure the temperature used is bounded
				uiTemperature = NTmin;
			if( uiTemperature > NTmax )
				uiTemperature = NTmax;

			xtoa( uiTemperature >> 1, uiWeightTable, cTempBuffer );
			LCDConstString2Spot( 2, 4, cBlank4 );
			LCDString2Spot( 2, 4, cTempBuffer );						//show the whole part of temperature
			LCDPutc( '.' );
			if( uiTemperature & 0x0001 )
				LCDPutc( '5' );
			else
				LCDPutc( '0' );

			if( uiTemperature >= (uiOldTemp + 2) )
			{
				ucDesiredPosition = 25 * (uiTemperature >> 1) - 50;		//calculate a new desired position
				uiOldTemp = uiTemperature;								//save a copy of this temp for next pass
				xtoa( uiOldTemp >> 1, uiWeightTable, cTempBuffer );
				LCDConstString2Spot( 1, 8, cBlank4 );
				LCDString2Spot( 1, 8, cTempBuffer );						//show the whole part of temperature
				LCDPutc( '.' );
				if( uiTemperature & 0x0001 )							//show the fractional part
					LCDPutc( '5' );
				else
					LCDPutc( '0' );
			}//end of if( uiTemperature >= (uiOldTemp + 2) )

//If the temperature is at NTmax for 5 consecutive passes, open the flap to max.
			if( uiTemperature >= NTmax )
			{
				ucTmaxCount++;
				if( ucTmaxCount >= 4 )									//5 consecutive passes yet?
				{
					ucDesiredPosition = 200;							//If Yes, force flap fully open.
					ucTmaxCount = 4;									//don't count passes past 4
				}
			}	
			else
				ucTmaxCount = 0;

			if( uiTemperature <= (uiOldTemp - 3) )
			{
				ucDesiredPosition = 25 * (uiTemperature >> 1) - 50;		//calculate a new desired position
				uiOldTemp = uiTemperature;								//save a copy of this temp for next pass
				xtoa( uiOldTemp >> 1, uiWeightTable, cTempBuffer );
				LCDConstString2Spot( 1, 8, cBlank4 );
				LCDString2Spot( 1, 8, cTempBuffer );					//show the whole part of old temperature
				LCDPutc( '.' );
				if( uiTemperature & 0x0001 )							//show the fractional part
					LCDPutc( '5' );
				else
					LCDPutc( '0' );
			}//end of if( uiTemperature <= (uiOldTemp - 3) )

//If the temperature is at NTmin for 5 passes, close the flap.
			if( uiTemperature <= NTmin )
			{
				ucTminCount++;
				if( ucTminCount >= 4 )									//5 consecutive passes yet?
				{
					ucDesiredPosition = 0;								//If Yes, force flap closed.
					ucTminCount = 4;									//don't count passes past 4
				}
			}	
			else
				ucTminCount = 0;
			
			ucMinsOld = ucMins;											//update the pass count

		}//end of if( ucMins != ucMinsOld )
	}//end of else( !ucModeFlag )
  }//The main loop ends here.
}//end of main





static void interrupt
ISR( void )
{

/*************************************************************************************************************************************************
		Timer 0 ISR: This interrupt happens every 16.384ms.
*************************************************************************************************************************************************/
	if( INTCON & 0x04 )												//Is TMR0IF set?
	{
		ucTicCount ++;
		ucTicFlag = 1;												//show that a tick has ocurred
		if( ucTicCount >= 62 )										// 61 * 16.384 ms = 0.999 sec
		{
			ucTicCount = 1;
			ucPass++;
		}
		if( ucPass > 59 )
		{
			ucMins++;
			ucPass = 0;
		}
		if( ucDesiredPosition > ucActualPosition )					//If Yes, need to go CCW.
		{
			PORTCbits.RC2 = 0;										//enable EasyDriver
			__delay_ms( 5 );
			PORTCbits.RC0 = 1;										//establish CW direction
			PORTCbits.RC1 = 1;										//generate a 
			PORTCbits.RC1 = 0;										//			step pulse
			PORTCbits.RC2 = 1;										//disable EasyDriver
			ucActualPosition++;
		}
		if( ucDesiredPosition < ucActualPosition )					//If Yes, need to go CW.
		{
			PORTCbits.RC2 = 0;										//enable EasyDriver
			__delay_ms( 5 );
			PORTCbits.RC0 = 0;										//establish CCW direction
			PORTCbits.RC1 = 1;										//generate a 
			PORTCbits.RC1 = 0;										//			step pulse
			PORTCbits.RC2 = 1;										//disable EasyDriver
			ucActualPosition--;
		}

		INTCON &= ~0x04;											//clear TMR0IF
	}

}
