#include	<htc.h>						//generic include for all devices
#include "LCD.h"

/*************************************************************************************************************************************************
	LCDConstString2Spot() writes a const string to a specific place on the LCD.
	Top line is 1, bottom line is 2. Spot is a number from 0d0 to 0d39.
*************************************************************************************************************************************************/
void LCDConstString2Spot(char cLine, char cSpot, const char *String)
{
	CursorXY( cSpot, cLine );
    PutConstString2LCD( String );
}


/*************************************************************************************************************************************************
	LCDString2Spot() writes a const string to a specific place on the LCD.
	Top line is 1, bottom line is 2. Spot is a number from 0d0 to 0d39.
*************************************************************************************************************************************************/
void LCDString2Spot(char cLine, char cSpot, char *String)
{
	CursorXY( cSpot, cLine );
    LCDPuts( String );
}


/*************************************************************************************************************************************************
		xtoa() is modelled after some code by Nicholas J. Conn - http://msp430launchpad.com
His code was far more cryptic than mine, and no comments of course. My version is more understandable.
------------------------------------------ Note: this code is designed only for unsigned ints. ---------------------------------------------------
*************************************************************************************************************************************************/
void xtoa( unsigned int uiNum, const unsigned int *Ptr, char * cBuff )
{
	unsigned char ucAscValue;
	unsigned int uiWeightValue;
	if( uiNum )
	{
		while( uiNum < *Ptr )                                	//to start with the weight table entry
			++Ptr;                                          	//must be smaller than the number to convert
		do
		{
			uiWeightValue = *Ptr++;                        	 	//get first useable entry from table
			ucAscValue = '0';                               	//start with the ascii value of '0'
			while( uiNum >= uiWeightValue )                 	//keep subtracting weight values from uiNum,
			{
				++ucAscValue;                               	//and adding 1 to the ascii value until
                uiNum -= uiWeightValue;                         //uiNum <= uiWeightValue
			}
			*cBuff++ = ucAscValue;                          	//store the ascii value in the table & move pointer
		}while( !(uiWeightValue & 1) );                     	//are we at the bottom of the table yet?
    *cBuff = 0;                                             	//null terminate the string when done, pointer was moved above
	}
	else
	{
		*cBuff++ = '0';                                       	//show '0' if the number passed in was zero
		*cBuff = 0;												//								null terminate the string
	}
}


/*************************************************************************************************************************************************
		LCDWrite() writes a byte to the LCD in 4 bit mode, RS must be preselected for commands(LCD_RS=0) or data(LCD_RS=1)
*************************************************************************************************************************************************/
void LCDWrite( unsigned char ucChar )
{
	__delay_us( 40 );
	LCD_DATA &= 0x0F;											//clear the upper nibble of the LCD data port
	LCD_DATA |= ( ucChar & 0xF0 );								//isolate MSN of char and OR it to data port
	LCD_STROBE();												//										and write it
	LCD_DATA &= 0x0F;											//clear the upper nibble of the LCD data port
	LCD_DATA |= ( ( ucChar << 4 ) & 0xF0 );						//shift LSN to MSN position and OR it to data port
	LCD_STROBE();												//										and write it
	__delay_us( 40 );
}


/*************************************************************************************************************************************************
											Clear and home the LCD
*************************************************************************************************************************************************/
void LCDClear(void)
{
	LCD_RS = 0;						// write commands
	LCDWrite( 0x01 );
	__delay_ms( 2 );
}


/*************************************************************************************************************************************************
										write a string of chars to the LCD
*************************************************************************************************************************************************/
void LCDPuts( char * String )
{
	LCD_RS = 1;						// write characters
	while( *String )
		LCDWrite( *String++ );
}


/*************************************************************************************************************************************************
										write a const string of chars to the LCD
*************************************************************************************************************************************************/
void PutConstString2LCD( const char * String )
{
	LCD_RS = 1;						// write characters
	while( *String )
		LCDWrite( *String++ );
}


/*************************************************************************************************************************************************
										write one character to the LCD
*************************************************************************************************************************************************/
void LCDPutc( char cChar )
{
	LCD_RS = 1;						// write character
	LCDWrite( cChar );
}


/*************************************************************************************************************************************************
			Move the cursor to the specified x(horizontal), y(line) position. y=1 top line, y=2 bottom line
*************************************************************************************************************************************************/
void CursorXY( unsigned char cX, unsigned char cY )
{
	unsigned char ucAddress;

	LCD_RS = 0;
	if( cY == 1 )							// If Yes, this is the 1st line
		ucAddress = 0x80;
	else
		ucAddress = 0xC0;					// If No, the must be 2nd line
		
	LCDWrite( ucAddress + cX );				// Combine cX with ucAddress to get actual address.
}


/*************************************************************************************************************************************************
									initialise the LCD into 4 bit mode
*************************************************************************************************************************************************/
void LCDInit()
{
	LCD_RS = 0;
	LCD_EN = 0;

	__delay_ms(15);							// wait 15mSec after power applied,
	LCD_DATA = 0x30;
	LCD_STROBE();
	__delay_ms(5);
	LCD_STROBE();
	__delay_us(200);
	LCD_STROBE();
	__delay_us(200);
	LCD_DATA = 0x20;						// Four bit mode
	LCD_STROBE();

	LCDWrite( 0x28 ); 						// set 4-bit data, 2-line display, 5x7 font
	LCDWrite( 0x0C ); 						// Display On, Cursor OFF, No Cursor Blink
//	LCDWrite( 0x0E ); 						// Display On, Cursor ON, No Cursor Blink
	LCDClear();								// Clear screen, cursor home
	LCDWrite( 0x06 ); 						// move cursor right
}


/*************************************************************************************************************************************************
	Byte2ascii converts the byte passed into 2 ascii bytes and shows it on the LCD at the current cursor position.
*************************************************************************************************************************************************/
void Byte2ascii( char cChar, char * cPtr )
{
	char cTemp;
	cTemp = cChar;
	cChar = cChar >> 4;							//isolate MSNibble
	*cPtr = Hex2ascii( cChar );					//				and store it
	cTemp &= 0x0F;								//isolate LSNibble
	*(cPtr + 1 ) = Hex2ascii( cTemp );			//				and store it

}



/*************************************************************************************************************************************************
		Hex2ascii converts the byte passed in to ascii and shows it on the LCD at the current cursor position.
*************************************************************************************************************************************************/
char Hex2ascii( char cChar )
{
	if( cChar < 10 )
		cChar += 0x30;							//for digits just add 0x30
	else if( cChar > 9 )
		cChar += 0x37;							//for letters just add 0x37
	return cChar;
}


/*************************************************************************************************************************************************
		Blink():	Blinks the symbol on the lcd.	
*************************************************************************************************************************************************/
void Blink( char ucColon )
{
	if(( ucColon & 0x01 ) == 0 )				//blank off all but LSBit
	{
		CursorXY( 15, 2 );						//position cursor
		LCDPutc( 0xF4 );							//				and show an omega
	}
	else
	{
		CursorXY( 15, 2 );						//position cursor
		LCDPutc( ' ' );							//				and show a blank
	}
}
